#include <stdio.h>
#include <string.h>
#include "discid.h"

#if defined(linux)
#include "linux/IoCtlDisc.h"
typedef CIoCtlDisc COSDisc; 
#elif defined(__APPLE__)
#include "mac/MacIoCtlDisc.h"
typedef CMacIoCtlDisc COSDisc; 
#else
#include "win32/MciDisc.h"
typedef CMciDisc COSDisc; 
#endif

#ifndef ERROR
#define ERROR (-1)
#endif

#ifndef OK
#define OK 0
#endif

const char *version = "discid 0.1.2";

static int cddb_sum(int n)
{
  int result = 0;
  while (n > 0)
  {
    result += n % 10;
    n /= 10;
  }

  return result;
}

static int discid(char *p_device)
{
  int status = 0;
  CDisc *p_disc = NULL;

  if (p_device)
    p_disc = new COSDisc(p_device);
  else
    p_disc = new COSDisc;

  if (p_disc)
  {
    unsigned track_count = p_disc->GetTrackCount();
    if (track_count > 0)
    {
      unsigned n = 0;
      for (unsigned track = 0; track < track_count; ++track)
        n += cddb_sum(p_disc->GetStartFrame(track) / FRAMES_PER_SECOND);

      unsigned start_sec = p_disc->GetStartFrame(0) / FRAMES_PER_SECOND;
      unsigned leadout_sec =
        (p_disc->GetStartFrame(track_count - 1) + p_disc->GetFrames(track_count - 1)) / FRAMES_PER_SECOND;
      unsigned total = leadout_sec - start_sec;

      unsigned id = ((n % 0xff) << 24 | total << 8 | track_count);

      printf("%08X %d", id, track_count);

      for (unsigned index = 0; index < track_count; ++index)
        printf(" %ld", p_disc->GetStartFrame(index));

      unsigned long leadout_frame =
        p_disc->GetStartFrame(track_count - 1) + p_disc->GetFrames(track_count - 1);
      printf(" %ld\n", leadout_frame / FRAMES_PER_SECOND);
    }
    else
      status = ERROR;
  }
  else
    status = ERROR;

  delete p_disc;

  return status;
}

int main(int argc, char *argv[])
{
  int status = OK;

  if (argc > 1)
  {
    for (int arg = 1; ((status == OK) && (arg < argc)); ++arg)
    {
      if (strncmp(argv[arg], "--", 2) == 0)
      {
        if (strcmp(argv[arg], "--help") == 0)
          fprintf(stderr, "Usage: %s [--help] [--version] [device...]\n", argv[0]);
        else if (strcmp(argv[arg], "--version") == 0)
          fprintf(stderr, "%s (%s) %s\n", version, VERSIONINFO, __DATE__);
        else
        {
          fprintf(stderr, "%s: unknown option '%s'.\nUse %s --help for help\n", argv[0], argv[arg], argv[0]);
          status = ERROR;
        }
      }
      else
        status = discid(argv[arg]); // device
    }
  }
  else // no arguments given
    status = discid(NULL); // default device

  return status;
}
